/** @file BackgroundImageStationSet.h
 * 
 * @author marco corvi
 * @date dec 2009
 *
 * @brief station positions for the background image
 * --------------------------------------------------------
 *  Copyright This sowftare is distributed under GPL-3.0 or later
 *  See the file COPYING.
 */
#ifndef BACKGROUND_IMAGE_STATION_SET_H
#define BACKGROUND_IMAGE_STATION_SET_H

#ifdef HAS_BACKIMAGE

#include <vector>

// #include <QGraphicsELlipseItem>

#include "BackgroundImageStation.h"

class BackgroundImageStationSet
{
  private:
    std::vector< BackgroundImageStation * > stations; //!< stations correspondences

  public:
    ~BackgroundImageStationSet()
    {
      clear();
    }

    void clear()
    {
      for (  std::vector< BackgroundImageStation * >::iterator it = stations.begin();
             it != stations.end();
             ++it ) {
        delete *it;
      }
      stations.clear();
    }

    /** add a new station point (correspondence)
     * @param name   station name
     * @param x      X coord of the point on the pixmap
     * @param y      Y coord of the point on the pixmap
     */
    void addStation( const QString & name, double x, double y, QGraphicsEllipseItem * p )
    { 
      stations.push_back( new BackgroundImageStation( name.TO_CHAR(), x, y, p) );
      // update();
    }

    /** get the station at a point (actually close to)
     * @param x   x coord
     * @param y   y coord
     * @return station or NULL
     */
    BackgroundImageStation * getStationAt( double x, double y )
    { 
      for ( std::vector< BackgroundImageStation * >::iterator sit = stations.begin(),
            end = stations.end();
            sit != end;
            ++sit ) {
        if ( abs(x - (*sit)->x) < 4 && abs(y - (*sit)->y) < 4 ) 
          return (*sit);
      }
      return NULL;
    }


    /** remove the station from the set (and delete it)
     * @param st   station pointer
     * @note must remove graphics item before calling this method
     */
    void removeStation( BackgroundImageStation * st )
    {
      for ( std::vector< BackgroundImageStation * >::iterator sit = stations.begin(),
             end = stations.end();
             sit != end;
             ++sit ) {
        if ( (*sit) == st ) {
           stations.erase( sit );
           delete st;
           break;
        }
      }
    }

    void changeStation( BackgroundImageStation * st, const char * name )
    {
      st->name = name;
    }
    
    /** get the vector of stations
     * @return a ref. to the vector of stations
     */
    const std::vector< BackgroundImageStation * > & getStations() const { return stations; }

};

#endif // HAS_BACKIMAGE

#endif

