/** @file CanvasUndo.h
 *
 * @author marco corvi
 * @date dec 2009
 *
 * @brief canvas undo list
 * --------------------------------------------------------
 *  Copyright This sowftare is distributed under GPL-3.0 or later
 *  See the file COPYING.
 */
#ifndef CANVAS_UNDO_H
#define CANVAS_UNDO_H

#define UNDO_POINT      0
#define UNDO_LINEPOINT  1
#define UNDO_ENDLINE    2
#define UNDO_CLOSELINE  3
#define UNDO_AREAPOINT  4
#define UNDO_FINISHAREA 5

/** undo struct
 * LIFO list of undo-commands
 * the undo are inserted at the head of the list and removed from the head
 */
class CanvasUndo
{
  private:
    int command;              //!< commandtype: 0 point, 1 line-point, 2 end-line
    struct CanvasUndo * next_undo; //!< link
 
  public:
    /** cstr
     * @param cmd   command type
     * @param n     next canvas-undo
     *
     * @note the cstr puts the new CanvasUndo at the head of th list of undo's
     */
    CanvasUndo( int cmd, CanvasUndo * next )
      : command( cmd )
      , next_undo( next )
    { }

    /** get the next undo
     * @return the next undo
     */
   CanvasUndo * next() const { return next_undo; }

   /** get the command
    * @return the undo command
    */
   int getCommand() const { return command; }

  
};

#endif // CANVAS_UNDO_H

