/** @file PlotAreaItem.h
 *
 * @author marco corvi
 * @date march 2010
 *
 * @brief Plot area item
 * --------------------------------------------------------
 *  Copyright This sowftare is distributed under GPL-3.0 or later
 *  See the file COPYING.
 */
#ifndef PLOT_AREA_ITEM_H
#define PLOT_AREA_ITEM_H


#include <QGraphicsPathItem>
#include <QGraphicsPolygonItem>

#include "IconSet.h"
#include "PlotZValue.h"
#include "TherionArea.h"

class PlotAreaItem : public QGraphicsPathItem 
                     // public QGraphicsPolygonItem
{
  private:
    QVector< QPointF > contour;

  public:
    PlotAreaItem( ThArea2D * area )
    {
      QPen   pen;
      QBrush brush;
      QColor color;
      IconSet::Get() -> AreaStyle( area->Type(), color, pen, brush );
      // setPen( pen );
      setBrush( brush );
      setZValue( ZVALUE_AREA );
      reset( area );
      area->setItem( this );
    }

    void reset( ThArea2D * area )
    {
      QPainterPath path;
      double x1, y1, x2, y2, x1c, y1c, x2c, y2c;
      ThLinePoint * prev = area->operator[](0);
      ThLinePoint * pt;
      path.moveTo( prev->x, prev->y );
      contour.clear();
      contour.append( QPointF(prev->x, prev->y) );
      for ( size_t k = 1; k < area->Size(); ++k ) {
        pt = area->operator[](k);
        contour.append( QPointF(pt->x, pt->y) );
        ThLine2D::computeCP( prev, pt, x1, y1, x1c, y1c, x2, y2, x2c, y2c );
        path.cubicTo( x1c, y1c, x2c, y2c, x2, y2 );
        prev = pt;
      }
      pt = area->operator[](0);
      ThLine2D::computeCP( prev, pt, x1, y1, x1c, y1c, x2, y2, x2c, y2c );
      path.cubicTo( x1c, y1c, x2c, y2c, x2, y2 );
      QPolygonF polygon( contour );
      // setPolygon( polygon );
      setPath( path );
    }

};

#endif

