/** @file PlotList.h
 *
 * @author marco corvi
 * @date feb 2011
 *
 * @brief list of plots
 * --------------------------------------------------------
 *  Copyright This sowftare is distributed under GPL-3.0 or later
 *  See the file COPYING.
 */
#ifndef PLOT_LIST_H
#define PLOT_LIST_H

#include <stdlib.h>

// #include <string>
#include <vector>
#include <utility>
#include <map>

#include <QString>

#include "shorthands.h"

class PlotCanvas; // forward


class PlotList
{
  private:
    std::vector< std::pair< QString, PlotCanvas *> > plots;
 
  public:
    PlotList()
    { }

    ~PlotList()
    {
      clear();
    }

    void clear();

    PlotCanvas * getPlot( const QString & name ) 
    {
      for ( std::vector< std::pair< QString, PlotCanvas *> >::iterator it = plots.begin(); 
            it != plots.end(); ++it ) {
        if ( it->first == name ) return it->second;
      }
      return NULL;
    }

    bool addPlot( const QString & pname, PlotCanvas * canvas ) 
    { 
      if ( getPlot( pname ) != NULL ) {
        fprintf(stderr, "Plot %s is already on the list\n", pname.TO_CHAR() );
        return false;
      }
      plots.push_back( std::pair< QString, PlotCanvas *> ( pname, canvas ) );
      return true;
    }

    bool removePlot( PlotCanvas * canvas, QString & name )
    {
      for ( std::vector< std::pair< QString, PlotCanvas *> >::iterator it = plots.begin(); 
            it != plots.end(); ++it ) {
        if ( it->second == canvas ) {
          name = it->first;
          plots.erase( it );
          return true;
        }
      }
      return false;
    }

    size_t plotsSize() const { return plots.size(); }

    const QString & getName( size_t k ) { return plots[k].first; }

    PlotCanvas * operator[] ( size_t k ) { return plots[k].second; }

  private:
    PlotList( PlotList const & );
    PlotList & operator= ( PlotList const & );

};
    

#endif

