/** @file PlotLineItem.h
 *
 * @author marco corvi
 * @date Jan 2010
 *
 * @brief plot line item
 * --------------------------------------------------------
 *  Copyright This sowftare is distributed under GPL-3.0 or later
 *  See the file COPYING.
 */
#ifndef PLOT_LINE_ITEM_H
#define PLOT_LINE_ITEM_H

#include <QGraphicsPathItem>
// #include <QGraphicsLineItem>

#include "PlotZValue.h"
#include "TherionLine.h"

// typedef QGraphicsLineItem PlotLineItem;

class PlotLineItem : public QGraphicsPathItem
{

  public:
    PlotLineItem() { }

    PlotLineItem( double x1, double y1, double x2, double y2, const QPen & pen )
    {
      setPen( pen );
      setZValue( ZVALUE_LINE );
      QPointF start( x1, y1 );
      QPainterPath path( start );
      path.lineTo( x2, y2 );
      setPath( path );
      show();
    }

    PlotLineItem( double x1, double y1, double x1c, double y1c, 
                  double x2c, double y2c, double x2, double y2, const QPen & pen )
    {
      setPen( pen );
      setZValue( ZVALUE_LINE );
      QPointF start( x1, y1 );
      QPainterPath path( start );
      path.cubicTo( x1c, y1c, x2c, y2c, x2, y2 );
      // path.closeSubpath();
      setPath( path );
      show();
    }

    void reset( ThLinePoint * p1, ThLinePoint * p2 )
    {
      double x1, y1, x1c, y1c, x2, y2, x2c, y2c;
      ThLine2D::computeCP( p1, p2, x1, y1, x1c, y1c, x2, y2, x2c, y2c );
      QPainterPath path;
      path.moveTo( x1, y1 );
      path.cubicTo( x1c, y1c, x2c, y2c, x2, y2 );
      setPath( path );
    }

};


#endif

