/** @file PlotPainter.h
 *
 * @author marco corvi
 * @date feb 2011
 *
 * @brief plot therion area
 * --------------------------------------------------------
 *  Copyright This sowftare is distributed under GPL-3.0 or later
 *  See the file COPYING.
 */
#ifndef PLOT_PAINTER_H
#define PLOT_PAINTER_H

#include <QPen>
#include <QGraphicsItem>
#include <QGraphicsLineItem>
#include <QGraphicsEllipseItem>


#include "IconSet.h"
#include "TherionLine.h"
#include "PlotLineItem.h"


class PlotPainter
{
   private:
     IconSet * icon;


   public:
     PlotPainter()
       : icon( IconSet::Get() )
     { }

     QGraphicsItem * createLinePoint( ThLinePoint * line_pt,  ThLinePoint * prev, Therion::LineType type );

     PlotLineItem * createLineSegment( ThLinePoint * from, ThLinePoint * to, Therion::LineType type );

     void createLineSegmentTo( ThLinePoint * from,
                               ThLinePoint * to,
                               Therion::LineType type,
                               QGraphicsItem ** ppt,
                               PlotLineItem ** psgm );

     void createAreaSegmentTo( ThLinePoint * from,
                               ThLinePoint * to,
                               Therion::AreaType type,
                               QGraphicsItem ** ppt,
                               PlotLineItem ** psgm );

     QGraphicsItem * createPoint( ThPoint2D * pt, double s );

     QGraphicsItem * createLabelPoint( ThPoint2D * pt, double s, const char * text, int orient );

   private:
     QGraphicsItem * createPolyPoint( double x, double y, const QPolygon & pts );

     QGraphicsEllipseItem * createRoundLinePoint( double x, double y, const QColor & color );

     QGraphicsEllipseItem * createRoundAreaPoint( double x, double y, const QColor & color );

     void rotatePenBrush( QPen & pen, double x1, double y1, double x2, double y2 );

     PlotLineItem * makeLine( ThLinePoint * from, ThLinePoint * to, const QPen & pen );

     QGraphicsItem * createAreaPoint( ThLinePoint * at, Therion::AreaType type );

     PlotLineItem * createAreaSegment( ThLinePoint * from, ThLinePoint * to, Therion::AreaType type );

};


#endif
