/** @file TherionArea.cpp
 *
 * @author marco corvi
 * @date feb 2011
 *
 * @brief plot therion area
 * --------------------------------------------------------
 *  Copyright This sowftare is distributed under GPL-3.0 or later
 *  See the file COPYING.
 */
#include "TherionArea.h"
#include "PlotAreaItem.h"

  
void 
ThArea2D::shift( double dx, double dy )
{
  for ( point_iterator it = Begin(), end = End(); it != end; ++it ) {
    (*it)->shift( dx, dy );
  }
  for ( segment_iterator it = SegmentBegin(), end = SegmentEnd(); it != end; ++it ) {
    (*it)->shift( dx, dy );
  }
  if ( item ) item->moveBy( dx*4, dy*4 );
}
  
 bool 
ThArea2D::Unfinish()
{
  if ( finished ) { // drop last segment
    DropLastSegment(); // FIX-2
    finished = false;
  }
  return finished;
}
  
bool 
ThArea2D::Finish( PlotLineItem * sgm_item )
{
  if ( ! finished ) {
    size_t s = pts.size();
    if ( s > 0 ) {
      InsertSegment(s-1, 0, sgm_item);
      finished = true;
    }
  }
  return finished;
}

void 
ThArea2D::addAreaPoint( ThLinePoint * pt,
                        QGraphicsItem * pt_item,
                        PlotLineItem * sgm_item )
{ 
  pts.push_back( pt );
  pt->item = pt_item;
  if ( sgm_item ) {
    size_t s = pts.size();
    InsertSegment(s-2, s-1, sgm_item);
  }
}

void 
ThArea2D::DropLastPoint()
{
  if ( pts.size() == 0 ) return;
  ThLinePoint * pt = pts.back();
  delete pt;
  pts.pop_back();
}
    
void 
ThArea2D::DropLastSegment()
{
  if ( sgms.size() == 0 ) return;
  ThLineSegment * sgm = sgms.back();
  delete sgm;
  sgms.pop_back(); // erase  from vector
}

void 
ThArea2D::InsertSegment( size_t s1, size_t s2, PlotLineItem * sgm_item )
{
  ThLinePoint * p2 = pts[s1];
  ThLinePoint * p1 = pts[s2];
  ThLineSegment * sgm = new ThLineSegment( p1, p2, sgm_item );
  sgms.push_back( sgm );
}

  

