/** @file ThLineType.cpp
 *
 * @author marco corvi
 * @date aug. 2009
 *
 * @brief therion line names
 * --------------------------------------------------------
 *  Copyright This sowftare is distributed under GPL-3.0 or later
 *  See the file COPYING.
 */
#include <stdlib.h>
#include <string.h>

#include "ThLineType.h"

const char * 
Therion::LineName[] = {
  "arrow",
  "border",
  "chimney",
  "pit",
  "user",
  "wall",
  "rock-border",
  "slope",
  "contour",
};

const char *
Therion::LineSubtype[THL_PLACEMARK][ THL_SUBTYPE_MAX ] = {
  { NULL, },
  { "visible", "invisible", "temporary", "presumed", NULL, },
  { NULL, },
  { NULL, },
  { NULL, },
  { "invisible", "bedrock", "sand", "clay", "pebbles", 
    "debris", "blocks", "ice", "underlying", "overlying",
    "unsurveyed", "presumed", "pit", "flowstone", "moonmilk", NULL },
  { NULL, },
  { NULL, },
  { NULL, },
};

/* Notes
type-specific options:
  pit, wall:pit : height <value>
  arrow : head <begin/end/both/none>
  contour : gradient <none/center/point>
  wall : altitude <value>
  slope : border <on/off>
  section : direction <begin/end/both/none/point>
 */

const char * 
Therion::LineSpecialOptionName[THL_PLACEMARK] = {
  "head",     // arrow
  NULL,       // border
  NULL,       // chimney
  "height",   // pit
  NULL,       // user
  "altitude", // wall
  NULL,       // rock-border
  "border",   // slope
  "gradient", // contour
};  

const char * 
Therion::LineSpecialOption[THL_PLACEMARK][THL_SPECIAL_MAX] = {
  { "begin", "end", "both", "none", NULL, },
  { NULL, },
  { NULL, },
  { "VALUE", NULL, },
  { NULL, },
  { "VALUE", NULL, },
  { NULL, },
  { "on", "off", NULL, },
  { "none", "center", "point", NULL, },
};

const char *
Therion::LineOutline[4] = { "none", "out", "in", NULL };


const char * 
Therion::LinePointAdjust[3] = { "horizontal", "vertical", NULL };

Therion::LineType 
Therion::getLineType( const char * type )
{
  for ( int t = 0; t < (int)THL_PLACEMARK; ++t ) {
    if ( strcmp( type, LineName[t] ) == 0 ) return (LineType)t;
  }
  return THL_PLACEMARK;
}

const char * 
Therion::getLineSubtype( LineType t, const char * st )
{
  const char ** ret = &( LineSubtype[t][0] );
  while ( *ret ) {
    if ( strcmp( *ret, st ) == 0 ) break;
    ++ *ret;
  }
  return *ret;
}

const char * 
Therion::getLinePointAdjust( const char * ad )
{
  const char ** ret = LinePointAdjust;
  while ( *ret ) {
    if ( strcmp( *ret, ad ) == 0 ) break;
    ++ *ret;
  }
  return *ret;
}

const char * 
Therion::getLineOutline( const char * ol )
{
  const char ** ret = &(LineOutline[0]);
  while ( *ret ) {
    if ( strcmp( *ret, ol ) == 0 ) break;
    ++ *ret;
  }
  return *ret;
}

const char * 
Therion::getLineSpecialOption( LineType t, const char * so )
{
  const char ** ret = &(LineSpecialOption[t][0]);
  while ( *ret ) {
    if ( strcmp( *ret, so ) == 0 ) break;
    ++ *ret;
  }
  return *ret;
}

