/** @file ThPointType.cpp
 *
 * @author marco corvi
 * @date feeb 2011
 *
 * @brief therion point names
 * --------------------------------------------------------
 *  Copyright This sowftare is distributed under GPL-3.0 or later
 *  See the file COPYING.
 */
#include <stdlib.h> // NULL
#include <string.h>

#include "ThPointType.h"

const char * 
Therion::PointName[] = {
  "air-draught",   // 0
  "anchor",
  "blocks",
  "breakdown-choke",
  "clay",

  "clay-choke",
  "continuation",
  "crystal",
  "curtain",
  "debris",

  "dig",
  "entrance",
  "flowstone",
  "flowstone-choke",
  "helictite",

  "ice",
  "label",         // 5
  "narrow-end",
  "pebbles",
  "pillar",

  "popcorn",
  "sand",
  "snow",
  "stalactite",
  "stalagmite",    // 10

  "user",
  "water-flow", 
  "station", // station must be last
};

const char *
Therion::PointGroupName[THPG_PLACEMARK] = {
  "air",
  "ends",
  "fills",
  "formations",
  "notes",
  "riggings",
  "user",
  "water",
  "station"
};

const int
Therion::PointGroup[THPG_PLACEMARK][10] = {
  { THP_AIR_DRAUGHT, -1, },
  { THP_BREAKDOWN_CHOKE, THP_CLAY_CHOKE, THP_FLOWSTONE_CHOKE,
    THP_NARROW_END, -1, },
  { THP_BLOCKS, THP_CLAY, THP_DEBRIS, THP_FLOWSTONE, THP_ICE,
    THP_PEBBLES, THP_SAND, THP_SNOW, -1, },
  { THP_CRYSTAL, THP_CURTAIN, THP_HELICTITE, THP_PILLAR, THP_POPCORN,
    THP_STALACTITE, THP_STALAGMITE, -1, },
  { THP_CONTINUATION, THP_ENTRANCE, THP_LABEL, -1, },
  { THP_ANCHOR, THP_DIG, -1, },
  { THP_USER, -1, },
  { THP_WATER_FLOW, -1, },
  { THP_STATION, -1, }
};

const bool
Therion::PointHasOrientation[] = {
  true, false, false, false, false,  // clay
  false, false, false, false, false, // debris
  false, true, false, false, false,  // helictite
  false, true, false, false, false,  // pillar
  false, false, false, false, false, // stalagmite
  false, true, false
};


const char * 
Therion::PointSubtype[ THP_PLACEMARK ][ THP_SUBTYPE_MAX ] = {
 { "undefined", "winter", "summer", NULL, },
 { NULL, }, // anchor
 { NULL, }, // blocks
 { NULL, }, // breakdown-choke
 { NULL, }, // clay

 { NULL, }, // clay-choke
 { NULL, }, // cont.
 { NULL, }, // crystal
 { NULL, }, // curtain
 { NULL, }, // debris

 { NULL, }, // dig
 { NULL, }, // entrance
 { NULL, }, // flowstone
 { NULL, }, // flowstone-choke
 { NULL, }, // helictite

 { NULL, }, // ice
 { NULL, }, // label
 { NULL, }, // narrow-end
 { NULL, }, // pebbles
 { NULL, }, // pillar

 { NULL, }, // popcorn
 { NULL, }, // sand
 { NULL, }, // snow
 { NULL, }, // stalactite
 { NULL, }, // stalagmite

 { "custom", NULL, }, // user
 { "permanent", "intermittent", "paleo", NULL, }, // water-flow
 { "temporary", "painted", "natural", "fixed", NULL }
};

const char * 
Therion::PointScale[ ] = {
  "tiny", "small", "normal", "large", "huge", 
  NULL 
};

const char * 
Therion::PointAlign[ ] = {
  "center", "top", "bottom", "left", "right", 
  "top-left", "top-right", "bottom-left", "bottom-right",
  NULL
};
  
Therion::PointType 
Therion::getPointType( const char * type )
{
  for (int t=0; t<(int)THP_PLACEMARK; ++t ) {
    if ( strcmp( type, PointName[t] ) == 0 ) return (PointType)t;
  }
  return THP_PLACEMARK;
}

const char * 
Therion::getPointSubtype( PointType t, const char * st )
{
  const char ** ret = &( PointSubtype[t][0] );
  while ( *ret ) {
    if ( strcmp( *ret, st ) == 0 ) break;
    ++ *ret;
  }
  return *ret;
}

const char * 
Therion::getPointScale( const char * sc ) 
{
  const char ** ret = &(PointScale[0]);
  while ( *ret ) {
    if ( strcmp( *ret, sc ) == 0 ) break;
    ++ *ret;
  }
  return *ret;
}

const char * 
Therion::getPointAlign( const char * al ) 
{
  const char ** ret = &(PointAlign[0]);
  while ( *ret ) {
    if ( strcmp( *ret, al ) == 0 ) break;
    ++ *ret;
  }
  return *ret;
}

