/** @file ThPointType.h
 *
 * @author marco corvi
 * @date aug. 2009
 *
 * @brief plot therion point types
 * --------------------------------------------------------
 *  Copyright This sowftare is distributed under GPL-3.0 or later
 *  See the file COPYING.
 */
#ifndef TH_POINT_TYPE_H
#define TH_POINT_TYPE_H

namespace Therion
{
  /** therion point types
   */
  enum PointType {
    THP_AIR_DRAUGHT = 0, // 0
    THP_ANCHOR,
    THP_BLOCKS,       // 2
    THP_BREAKDOWN_CHOKE,
    THP_CLAY,         // 4
    THP_CLAY_CHOKE,
    THP_CONTINUATION, // 6
    THP_CRYSTAL,
    THP_CURTAIN,
    THP_DEBRIS,       // 9
    THP_DIG,
    THP_ENTRANCE,     // 11
    THP_FLOWSTONE,
    THP_FLOWSTONE_CHOKE,
    THP_HELICTITE,
    THP_ICE,          // 15
    THP_LABEL,        // 16
    THP_NARROW_END,
    THP_PEBBLES,      // 18
    THP_PILLAR,
    THP_POPCORN,
    THP_SAND,         // 21
    THP_SNOW,
    THP_STALACTITE,   // 23
    THP_STALAGMITE,   // 24
    THP_USER,         // 25
    THP_WATER_FLOW,   // 26
    // station must be last
    THP_STATION,      
    THP_PLACEMARK
  };

  enum PointGroupType {
    THPG_AIR,      
    THPG_END,
    THPG_FILL,
    THPG_FORMATION,
    THPG_NOTE,
    THPG_RIG,       
    THPG_USER,
    THPG_WATER,
    THPG_STATION,
    THPG_PLACEMARK
  };

  PointType getPointType( const char * type );

  extern const char * PointGroupName[THPG_PLACEMARK];

  extern const int PointGroup[THPG_PLACEMARK][10]; // FIXME bound

  /** whether the point can have orientation
   */
  extern const bool PointHasOrientation[ THP_PLACEMARK ];

  #define THP_SUBTYPE_MAX 5
  extern const char * PointSubtype[ THP_PLACEMARK ][ THP_SUBTYPE_MAX ];
  
  extern const char * PointName[ THP_PLACEMARK ];

  #define THP_ALIGN_MAX 10
  extern const char * PointAlign[ THP_ALIGN_MAX ];

  #define THP_SCALE_MAX 6
  extern const char * PointScale[ THP_SCALE_MAX ];

  const char * getPointAlign( const char * al );
  const char * getPointScale( const char * sc );
  const char * getPointSubtype( PointType t, const char * st );

}

#endif
